@@ -81,7 +81,7 @@ class KeyExchange(enum.Enum):
         forward_secret=False
     )
     DH = KeyExchangeParams(  # pylint: disable=invalid-name
-        name='Diffie–Hellman Ephemeral (DHE)',
+        name='static Diffie–Hellman Ephemeral (DH)',
         forward_secret=False
     )
     DHE = KeyExchangeParams(
@@ -89,7 +89,7 @@ class KeyExchange(enum.Enum):
         forward_secret=True
     )
     ECDH = KeyExchangeParams(
-        name='Elliptic-curve Diffie–Hellman (ECDH)',
+        name='static Elliptic-curve Diffie–Hellman (ECDH)',
         forward_secret=False
     )
     ECDHE = KeyExchangeParams(
@@ -618,6 +618,11 @@ class Hash(AlgortihmOIDBase, enum.Enum):
         oid='1.3.101.115',
         digest_size=448
     )
+    WHIRLPOOL = HashParams(
+        name='Whirlpool',
+        oid='1.0.10118.3.0.55',
+        digest_size=512
+    )
 
 
 @attr.s(frozen=True)
@@ -835,6 +840,11 @@ class MAC(AlgortihmOIDBase, enum.Enum):
         oid='1.3.101.115',
         hash_algo=Hash.ED448PH
     )
+    WHIRLPOOL = HMACParams(
+        name='Whirlpool',
+        oid=None,
+        hash_algo=Hash.WHIRLPOOL
+    )
 
 
 class NamedGroupType(enum.IntEnum):
