@@ -11,12 +11,14 @@ class InvalidDataLength(Exception):
 
 @attr.s
 class NotEnoughData(InvalidDataLength):
-    pass
+    def __str__(self):
+        return 'not enough data received from target; missing_byte_count="{}"'.format(self.bytes_needed)
 
 
 @attr.s
 class TooMuchData(InvalidDataLength):
-    pass
+    def __str__(self):
+        return 'too much data received from target; rest_byte_count="{}"'.format(self.bytes_needed)
 
 
 @attr.s(init=False)
@@ -29,10 +31,12 @@ class InvalidValue(Exception):
         elif isinstance(value, int):
             message = hex(value)
         else:
-            message = '{}'.format(value)
-        message = '{} is not a valid {}'.format(message, type_class.__name__)
+            message = value
+        message = hex(value) if isinstance(value, int) else repr(value)
+        type_name = type_class.__name__ if hasattr(type_class, '__name__') else str(type(type_class))
+        message = u'{} is not a valid {}'.format(message, type_name)
         if class_member is not None:
-            message = '{} {} value'.format(message, class_member)
+            message = u'{} {} value'.format(message, class_member)
 
         super(InvalidValue, self).__init__(message)
 
@@ -40,4 +44,5 @@ class InvalidValue(Exception):
 
 
 class InvalidType(Exception):
-    pass
+    def __str__(self):
+        return 'invalid type value received from target'
