@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import abc
+import datetime
 import enum
 import struct
 
@@ -8,6 +9,9 @@ import attr
 import six
 from six.moves import collections_abc
 
+import dateutil
+import dateutil.parser
+
 from cryptoparser.common.exception import NotEnoughData, TooMuchData, InvalidValue
 import cryptoparser.common.utils
 
@@ -54,6 +58,19 @@ class ParsableBase(ParsableBaseNoABC):
         raise NotImplementedError()
 
 
+class ParserCRLF(ParsableBase):
+    @classmethod
+    def _parse(cls, parsable):
+        parser = ParserText(parsable)
+
+        parser.parse_string('crlf', '\r\n')
+
+        return ParserCRLF(), 2
+
+    def compose(self):
+        return b'\r\n'
+
+
 _SIZE_TO_FORMAT = {
     1: 'B',
     2: 'H',
@@ -150,6 +167,8 @@ class ParserText(ParserBase):
             min_count,
             max_count
     ):
+        separators = separators.encode(self._encoding)
+
         count = 0
         actual_offset = count_offset
         while actual_offset < len(self._parsable) and self._parsable[actual_offset:actual_offset + 1] in separators:
@@ -159,13 +178,12 @@ class ParserText(ParserBase):
             if max_count is not None and count > max_count:
                 raise InvalidValue(self._parsable[count_offset:], type(self), name)
 
-        if count < min_count:
+        if min_count is not None and count < min_count:
             raise InvalidValue(self._parsable[count_offset:], type(self), name)
 
         return actual_offset - count_offset
 
     def parse_separator(self, separator, min_length=1, max_length=1):
-        separator = bytearray(separator, self._encoding)
         self._parsed_length += self._check_separators(
             'separator', self._parsed_length, separator, min_length, max_length
         )
@@ -195,16 +213,18 @@ class ParserText(ParserBase):
 
             last_item_offset = item_offset
 
-        self._parsed_length = item_offset
-        self._parsed_values[name] = value
+        return value, item_offset - self._parsed_length
 
     def parse_numeric(self, name, converter=int):
-        self._parse_numeric_array(name, 1, None, converter)
-        self._parsed_values[name] = self._parsed_values[name][0]
+        value, parsed_length = self._parse_numeric_array(name, 1, None, converter)
+        self._parsed_values[name] = value[0]
+        self._parsed_length += parsed_length
 
     def parse_numeric_array(self, name, item_num, separator, converter=int):
-        separator = bytearray(separator, self._encoding)
-        self._parse_numeric_array(name, item_num, separator, converter)
+        value, parsed_length = self._parse_numeric_array(name, item_num, separator, converter)
+
+        self._parsed_values[name] = value
+        self._parsed_length += parsed_length
 
     def parse_string(self, name, value):
         min_length = len(value)
@@ -264,34 +284,51 @@ class ParserText(ParserBase):
             separators,
             item_class,
             fallback_class,
-            may_end=False
+            may_end=False,
+            separator_spaces=''
     ):
-        for item_end in range(item_offset, len(self._parsable)):
-            if self._parsable[item_end] in separators:
+        item_end = None
+        byte_separators = [separator.encode(self._encoding) for separator in separators]
+
+        for separator_end in range(item_offset, len(self._parsable) + 1):
+            for separator in byte_separators:
+                if self._parsable[item_offset:separator_end].endswith(separator):
+                    item_end = separator_end - len(separator)
+                    break
+            if item_end is not None:
                 break
         else:
             if not may_end:
-                raise InvalidValue(self._parsable[item_offset:], type(self), name)
+                raise InvalidValue(self._parsable[item_offset:], type(item_class), name)
 
             item_end = len(self._parsable)
 
-        item = self._apply_item_class(name, item_offset, item_end, separators, item_class, fallback_class, may_end)
+        separator_space_count = 0
+        byte_separator_spaces = separator_spaces.encode(self._encoding)
+        while (item_end > item_offset and
+                self._parsable[
+                    item_end - separator_space_count - 1:
+                    item_end - separator_space_count
+                ] in byte_separator_spaces):
+            separator_space_count += 1
 
-        return item, item_end - item_offset
+        item = self._apply_item_class(
+            name, item_offset, item_end - separator_space_count, separators, item_class, fallback_class, may_end
+        )
 
-    def parse_string_until_separator(self, name, separator, item_class=str, fallback_class=None):
-        separator = bytearray(separator, self._encoding)
+        return item, item_end - item_offset - separator_space_count
+
+    def parse_string_until_separator(self, name, separators, item_class=str, fallback_class=None):
         parsed_value, parsed_length = self._parse_string_until_separator(
-            name, self._parsed_length, separator, item_class, fallback_class, False
+            name, self._parsed_length, separators, item_class, fallback_class, False
         )
 
         self._parsed_values[name] = parsed_value
         self._parsed_length += parsed_length
 
-    def parse_string_until_separator_or_end(self, name, separator, item_class=str, fallback_class=None):
-        separator = bytearray(separator, self._encoding)
+    def parse_string_until_separator_or_end(self, name, separators, item_class=str, fallback_class=None):
         parsed_value, parsed_length = self._parse_string_until_separator(
-            name, self._parsed_length, separator, item_class, fallback_class, True
+            name, self._parsed_length, separators, item_class, fallback_class, True
         )
 
         self._parsed_values[name] = parsed_value
@@ -309,23 +346,21 @@ class ParserText(ParserBase):
     ):  # pylint: disable=too-many-arguments
         value = []
         item_offset = self._parsed_length
-        separator = separator.encode(self._encoding)
-        separator_spaces = separator_spaces.encode(self._encoding)
         max_separator_count = None if skip_empty else 1
 
         if separator_spaces:
-            item_offset += self._check_separators('separator', item_offset, separator_spaces, 0, None)
+            item_offset += self._check_separators('separator', item_offset, separator_spaces, None, None)
 
         while True:
             parsed_value, parsed_length = self._parse_string_until_separator(
-                name, item_offset, separator + separator_spaces, str, None, True
+                name, item_offset, separator, str, None, True, separator_spaces
             )
             if parsed_length:
                 parsed_value = self._apply_item_class(
                     name,
                     item_offset,
                     item_offset + parsed_length,
-                    separator + separator_spaces,
+                    separator,
                     item_class,
                     fallback_class,
                     True
@@ -335,14 +370,15 @@ class ParserText(ParserBase):
             elif not skip_empty:
                 raise InvalidValue(self._parsable[item_offset:], type(self), name)
 
+            if separator_spaces:
+                item_offset += self._check_separators('separator', item_offset, separator_spaces, None, None)
+
             if item_offset == len(self._parsable):
                 break
 
-            if separator_spaces:
-                item_offset += self._check_separators('separator', item_offset, separator_spaces, 0, None)
             item_offset += self._check_separators(name, item_offset, separator, 1, max_separator_count)
             if separator_spaces:
-                item_offset += self._check_separators('separator', item_offset, separator_spaces, 0, None)
+                item_offset += self._check_separators('separator', item_offset, separator_spaces, None, None)
 
             if item_offset == len(self._parsable):
                 break
@@ -372,6 +408,27 @@ class ParserText(ParserBase):
             skip_empty=skip_empty
         )
 
+    def parse_date_time(self, name):
+        try:
+            value = self._parsable[self._parsed_length:]
+            date_time = dateutil.parser.parse(value.decode(self._encoding))
+        except ValueError as e:
+            six.raise_from(InvalidValue(value, type(self), 'value'), e)
+
+        self._parsed_values[name] = date_time
+        self._parsed_length = len(self._parsable)
+
+    def parse_time_delta(self, name):
+        value, parsed_length = self._parse_numeric_array(name, 1, None, int)
+
+        try:
+            time_delta = datetime.timedelta(seconds=value[0])
+        except OverflowError as e:
+            six.raise_from(InvalidValue(value[0], type(self), 'value'), e)
+
+        self._parsed_values[name] = time_delta
+        self._parsed_length += parsed_length
+
 
 @attr.s
 class ParserBinary(ParserBase):
@@ -473,7 +530,7 @@ class ParserBinary(ParserBase):
         except InvalidValue as e:
             raise e
 
-    def _parse_parsable_derived_array(self, name, items_size, item_classes, fallback_class=None):
+    def _parse_parsable_derived_array(self, items_size, item_classes, fallback_class=None):
         if items_size > self.unparsed_length:
             raise NotEnoughData(bytes_needed=items_size - self.unparsed_length)
 
@@ -496,26 +553,62 @@ class ParserBinary(ParserBase):
             unparsed_bytes = unparsed_bytes[parsed_length:]
             items.append(item)
 
-        self._parsed_values[name] = items
-        self._parsed_length += items_size
+        return items, items_size
 
     def parse_parsable_array(self, name, items_size, item_class, fallback_class=None):
         try:
-            self._parse_parsable_derived_array(name, items_size, [item_class, ], fallback_class)
+            items, items_size = self._parse_parsable_derived_array(items_size, [item_class, ], fallback_class)
         except NotEnoughData as e:
             raise e
         except ValueError as e:
             six.raise_from(InvalidValue(e.args[0], item_class, name), e)
 
+        self._parsed_values[name] = items
+        self._parsed_length += items_size
+
     def parse_parsable_derived_array(self, name, items_size, item_base_class, fallback_class=None):
         item_classes = cryptoparser.common.utils.get_leaf_classes(item_base_class)
         try:
-            self._parse_parsable_derived_array(name, items_size, item_classes, fallback_class)
+            items, items_size = self._parse_parsable_derived_array(items_size, item_classes, fallback_class)
         except NotEnoughData as e:
             raise e
         except ValueError as e:
             six.raise_from(InvalidValue(e.args[0], item_base_class, name), e)
 
+        self._parsed_values[name] = items
+        self._parsed_length += items_size
+
+    def _remove_trailing_separator(self, items, name):
+        if not isinstance(items[-1], ParserCRLF):
+            raise InvalidValue(self.unparsed, type(self), name)
+        del items[-1]
+
+    def parse_parsable_list(self, name, item_class, fallback_class=None, separator_class=ParserCRLF):
+        try:
+            items, items_size = self._parse_parsable_derived_array(
+                self.unparsed_length, [separator_class, item_class], fallback_class
+            )
+        except ValueError as e:
+            six.raise_from(InvalidValue(e.args[0], item_class, name), e)
+
+        if not items:
+            raise NotEnoughData(2)
+        self._remove_trailing_separator(items, name)
+
+        real_items = []
+        while items:
+            if len(items) % 2 == 0:
+                self._remove_trailing_separator(items, name)
+            else:
+                if isinstance(items[-1], separator_class):
+                    raise InvalidValue(self.unparsed, type(self), name)
+
+                real_items.insert(0, items[-1])
+                del items[-1]
+
+        self._parsed_values[name] = real_items
+        self._parsed_length += items_size
+
     def parse_variant(self, name, variant):
         parsed_object, value_length = variant.parse(self._parsable[self._parsed_length:])
 
@@ -589,6 +682,12 @@ class ComposerText(ComposerBase):
     def compose_separator(self, value):
         self.compose_string(value)
 
+    def compose_date_time(self, value, fmt):
+        self.compose_string(value.strftime(fmt))
+
+    def compose_time_delta(self, value):
+        self.compose_numeric(int(value.total_seconds()))
+
 
 @attr.s
 class ComposerBinary(ComposerBase):
@@ -637,8 +736,8 @@ class ComposerBinary(ComposerBase):
             self.compose_numeric(len(composed), item_size)
         self._composed += composed
 
-    def compose_parsable_array(self, values):
-        self._composed += bytearray().join(map(lambda item: item.compose(), values))
+    def compose_parsable_array(self, values, separator=bytearray()):
+        self._composed += separator.join(map(lambda item: item.compose(), values))
 
     def compose_bytes(self, value, item_size, converter=bytearray):
         value_bytes = converter(value)
